package sun.awt.shell;

import java.util.Comparator;
import sun.java2d.DisposerRecord;
import java.util.HashMap;
import java.util.Collections;
import java.util.List;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.io.ObjectStreamException;
import java.io.File;
import sun.java2d.Disposer;
import java.util.concurrent.Callable;
import java.io.IOException;
import java.util.Map;
import java.awt.Image;

final class Win32ShellFolder2 extends ShellFolder
{
    public static final int DESKTOP = 0;
    public static final int INTERNET = 1;
    public static final int PROGRAMS = 2;
    public static final int CONTROLS = 3;
    public static final int PRINTERS = 4;
    public static final int PERSONAL = 5;
    public static final int FAVORITES = 6;
    public static final int STARTUP = 7;
    public static final int RECENT = 8;
    public static final int SENDTO = 9;
    public static final int BITBUCKET = 10;
    public static final int STARTMENU = 11;
    public static final int DESKTOPDIRECTORY = 16;
    public static final int DRIVES = 17;
    public static final int NETWORK = 18;
    public static final int NETHOOD = 19;
    public static final int FONTS = 20;
    public static final int TEMPLATES = 21;
    public static final int COMMON_STARTMENU = 22;
    public static final int COMMON_PROGRAMS = 23;
    public static final int COMMON_STARTUP = 24;
    public static final int COMMON_DESKTOPDIRECTORY = 25;
    public static final int APPDATA = 26;
    public static final int PRINTHOOD = 27;
    public static final int ALTSTARTUP = 29;
    public static final int COMMON_ALTSTARTUP = 30;
    public static final int COMMON_FAVORITES = 31;
    public static final int INTERNET_CACHE = 32;
    public static final int COOKIES = 33;
    public static final int HISTORY = 34;
    public static final int ATTRIB_CANCOPY = 1;
    public static final int ATTRIB_CANMOVE = 2;
    public static final int ATTRIB_CANLINK = 4;
    public static final int ATTRIB_CANRENAME = 16;
    public static final int ATTRIB_CANDELETE = 32;
    public static final int ATTRIB_HASPROPSHEET = 64;
    public static final int ATTRIB_DROPTARGET = 256;
    public static final int ATTRIB_LINK = 65536;
    public static final int ATTRIB_SHARE = 131072;
    public static final int ATTRIB_READONLY = 262144;
    public static final int ATTRIB_GHOSTED = 524288;
    public static final int ATTRIB_HIDDEN = 524288;
    public static final int ATTRIB_FILESYSANCESTOR = 268435456;
    public static final int ATTRIB_FOLDER = 536870912;
    public static final int ATTRIB_FILESYSTEM = 1073741824;
    public static final int ATTRIB_HASSUBFOLDER = Integer.MIN_VALUE;
    public static final int ATTRIB_VALIDATE = 16777216;
    public static final int ATTRIB_REMOVABLE = 33554432;
    public static final int ATTRIB_COMPRESSED = 67108864;
    public static final int ATTRIB_BROWSABLE = 134217728;
    public static final int ATTRIB_NONENUMERATED = 1048576;
    public static final int ATTRIB_NEWCONTENT = 2097152;
    public static final int SHGDN_NORMAL = 0;
    public static final int SHGDN_INFOLDER = 1;
    public static final int SHGDN_INCLUDE_NONFILESYS = 8192;
    public static final int SHGDN_FORADDRESSBAR = 16384;
    public static final int SHGDN_FORPARSING = 32768;
    Win32ShellFolder2.FolderDisposer disposer;
    private long pIShellIcon;
    private String folderType;
    private String displayName;
    private Image smallIcon;
    private Image largeIcon;
    private Boolean isDir;
    private boolean isPersonal;
    private volatile Boolean cachedIsFileSystem;
    private volatile Boolean cachedIsLink;
    private static Map smallSystemImages;
    private static Map largeSystemImages;
    private static Map smallLinkedSystemImages;
    private static Map largeLinkedSystemImages;
    private static final int LVCFMT_LEFT = 0;
    private static final int LVCFMT_RIGHT = 1;
    private static final int LVCFMT_CENTER = 2;
    
    private static native void initIDs();
    
    private void setIShellFolder(long piShellFolder) {
        this.disposer.pIShellFolder = piShellFolder;
    }
    
    private void setRelativePIDL(long relativePIDL) {
        this.disposer.relativePIDL = relativePIDL;
    }
    
    private static String composePathForCsidl(int i) throws IOException, InterruptedException {
        String fileSystemPath = getFileSystemPath(i);
        return (fileSystemPath == null) ? ("ShellFolder: 0x" + Integer.toHexString(i)) : fileSystemPath;
    }
    
    Win32ShellFolder2(final int n) throws IOException, InterruptedException {
        super(null, composePathForCsidl(n));
        this.disposer = new Win32ShellFolder2.FolderDisposer();
        this.pIShellIcon = -1L;
        this.folderType = null;
        this.displayName = null;
        this.smallIcon = null;
        this.largeIcon = null;
        this.isDir = null;
        ShellFolder.invoke(new Callable<Void>() {
            @Override
            public Void call() throws InterruptedException {
                if (n == 0) {
                    Win32ShellFolder2.access$100(Win32ShellFolder2.this);
                }
                else {
                    Win32ShellFolder2.access$300(Win32ShellFolder2.this, Win32ShellFolder2.access$200(Win32ShellFolder2.this.getDesktop()), n);
                    long n = Win32ShellFolder2.this.disposer.relativePIDL;
                    Win32ShellFolder2.this.parent = Win32ShellFolder2.this.getDesktop();
                    while (n != 0L) {
                        long copyFirstPIDLEntry = Win32ShellFolder2.copyFirstPIDLEntry(n);
                        if (copyFirstPIDLEntry != 0L) {
                            n = Win32ShellFolder2.getNextPIDLEntry(n);
                            if (n != 0L) {
                                Win32ShellFolder2.this.parent = new Win32ShellFolder2((Win32ShellFolder2)Win32ShellFolder2.this.parent, copyFirstPIDLEntry);
                            }
                            else {
                                Win32ShellFolder2.this.disposer.relativePIDL = copyFirstPIDLEntry;
                            }
                        }
                        else {
                            break;
                        }
                    }
                }
                return null;
            }
        }, InterruptedException.class);
        Disposer.addRecord(this, this.disposer);
    }
    
    Win32ShellFolder2(Win32ShellFolder2 win32ShellFolder2, long piShellFolder, long relativePIDL, String s) {
        super(win32ShellFolder2, (s != null) ? s : "ShellFolder: ");
        this.disposer = new Win32ShellFolder2.FolderDisposer();
        this.pIShellIcon = -1L;
        this.folderType = null;
        this.displayName = null;
        this.smallIcon = null;
        this.largeIcon = null;
        this.isDir = null;
        this.disposer.pIShellFolder = piShellFolder;
        this.disposer.relativePIDL = relativePIDL;
        Disposer.addRecord(this, this.disposer);
    }
    
    Win32ShellFolder2(final Win32ShellFolder2 win32ShellFolder2, final long relativePIDL) throws InterruptedException {
        super(win32ShellFolder2, (String)ShellFolder.invoke(new Callable<String>() {
            @Override
            public String call() {
                return Win32ShellFolder2.access$400(Win32ShellFolder2.access$200(win32ShellFolder2), relativePIDL);
            }
        }, RuntimeException.class));
        this.disposer = new Win32ShellFolder2.FolderDisposer();
        this.pIShellIcon = -1L;
        this.folderType = null;
        this.displayName = null;
        this.smallIcon = null;
        this.largeIcon = null;
        this.isDir = null;
        this.disposer.relativePIDL = relativePIDL;
        Disposer.addRecord(this, this.disposer);
    }
    
    private native void initDesktop();
    
    private native void initSpecial(long p0, int p1);
    
    public void setIsPersonal() {
        this.isPersonal = true;
    }
    
    @Override
    protected Object writeReplace() throws ObjectStreamException {
        return ShellFolder.invoke(new Callable<File>() {
            @Override
            public File call() {
                if (Win32ShellFolder2.this.isFileSystem()) {
                    return new File(Win32ShellFolder2.this.getPath());
                }
                Win32ShellFolder2 drives = Win32ShellFolderManager2.getDrives();
                if (drives != null) {
                    File[] listFiles = drives.listFiles();
                    if (listFiles != null) {
                        for (int i = 0; i < listFiles.length; ++i) {
                            if ((listFiles[i] instanceof Win32ShellFolder2)) {
                                Win32ShellFolder2 win32ShellFolder2 = (Win32ShellFolder2)listFiles[i];
                                if (win32ShellFolder2.isFileSystem() && !win32ShellFolder2.hasAttribute(33554432)) {
                                    return new File(win32ShellFolder2.getPath());
                                }
                            }
                        }
                    }
                }
                return new File("C:\\");
            }
        });
    }
    
    protected void dispose() {
        this.disposer.dispose();
    }
    
    static native long getNextPIDLEntry(long p0);
    
    static native long copyFirstPIDLEntry(long p0);
    
    private static native long combinePIDLs(long p0, long p1);
    
    static native void releasePIDL(long p0);
    
    private static native void releaseIShellFolder(long p0);
    
    private long getIShellFolder() {
        if (this.disposer.pIShellFolder == 0L) {
            try {
                this.disposer.pIShellFolder = (long)ShellFolder.invoke(new Callable<Long>() {
                    @Override
                    public Long call() {
                        assert !(!Win32ShellFolder2.this.isDirectory());
                        assert Win32ShellFolder2.this.parent != null;
                        long parentIShellFolder = Win32ShellFolder2.this.getParentIShellFolder();
                        if (parentIShellFolder == 0L) {
                            throw new InternalError("Parent IShellFolder was null for " + Win32ShellFolder2.this.getAbsolutePath());
                        }
                        long access$500 = Win32ShellFolder2.access$500(parentIShellFolder, Win32ShellFolder2.this.disposer.relativePIDL);
                        if (access$500 == 0L) {
                            throw new InternalError("Unable to bind " + Win32ShellFolder2.this.getAbsolutePath() + " to parent");
                        }
                        return Long.valueOf(access$500);
                    }
                }, RuntimeException.class);
            }
            catch (InterruptedException ex) {}
        }
        return this.disposer.pIShellFolder;
    }
    
    public long getParentIShellFolder() {
        Win32ShellFolder2 win32ShellFolder2 = (Win32ShellFolder2)this.getParentFile();
        if (win32ShellFolder2 == null) {
            return this.getIShellFolder();
        }
        return win32ShellFolder2.getIShellFolder();
    }
    
    public long getRelativePIDL() {
        if (this.disposer.relativePIDL == 0L) {
            throw new InternalError("Should always have a relative PIDL");
        }
        return this.disposer.relativePIDL;
    }
    
    private long getAbsolutePIDL() {
        if (this.parent == null) {
            return this.getRelativePIDL();
        }
        if (this.disposer.absolutePIDL == 0L) {
            this.disposer.absolutePIDL = combinePIDLs(((Win32ShellFolder2)this.parent).getAbsolutePIDL(), this.getRelativePIDL());
        }
        return this.disposer.absolutePIDL;
    }
    
    public Win32ShellFolder2 getDesktop() {
        return Win32ShellFolderManager2.getDesktop();
    }
    
    public long getDesktopIShellFolder() {
        return this.getDesktop().getIShellFolder();
    }
    
    private static boolean pathsEqual(String s, String anotherString) {
        return s.equalsIgnoreCase(anotherString);
    }
    
    @Override
    public boolean equals(Object obj) {
        if (obj == null || !(obj instanceof Win32ShellFolder2)) {
            if (!(obj instanceof File)) {
                return super.equals(obj);
            }
            return pathsEqual(this.getPath(), ((File)obj).getPath());
        }
        else {
            Win32ShellFolder2 win32ShellFolder2 = (Win32ShellFolder2)obj;
            if ((this.parent == null && win32ShellFolder2.parent != null) || (this.parent != null && win32ShellFolder2.parent == null)) {
                return false;
            }
            if (this.isFileSystem() && win32ShellFolder2.isFileSystem()) {
                return pathsEqual(this.getPath(), win32ShellFolder2.getPath()) && (this.parent == win32ShellFolder2.parent || this.parent.equals(win32ShellFolder2.parent));
            }
            if (this.parent != win32ShellFolder2.parent && !this.parent.equals(win32ShellFolder2.parent)) {
                return false;
            }
            try {
                return pidlsEqual(this.getParentIShellFolder(), this.disposer.relativePIDL, win32ShellFolder2.disposer.relativePIDL);
            }
            catch (InterruptedException ex) {
                return false;
            }
        }
    }
    
    private static boolean pidlsEqual(final long n, final long n2, final long n3) throws InterruptedException {
        return (boolean)ShellFolder.invoke(new Callable<Boolean>() {
            @Override
            public Boolean call() {
                return Boolean.valueOf(Win32ShellFolder2.access$600(n, n2, n3) == 0);
            }
        }, RuntimeException.class);
    }
    
    private static native int compareIDs(long p0, long p1, long p2);
    
    @Override
    public boolean isFileSystem() {
        if (this.cachedIsFileSystem == null) {
            this.cachedIsFileSystem = Boolean.valueOf(this.hasAttribute(1073741824));
        }
        return (boolean)this.cachedIsFileSystem;
    }
    
    public boolean hasAttribute(final int n) {
        Boolean b = (Boolean)ShellFolder.invoke(new Callable<Boolean>() {
            @Override
            public Boolean call() {
                return Boolean.valueOf((Win32ShellFolder2.access$700(Win32ShellFolder2.this.getParentIShellFolder(), Win32ShellFolder2.this.getRelativePIDL(), n) & n) != 0x0);
            }
        });
        return b != null && (boolean)b;
    }
    
    private static native int getAttributes0(long p0, long p1, int p2);
    
    private static String getFileSystemPath(long n, long n2) {
        int n3 = 536936448;
        if (n == Win32ShellFolderManager2.getNetwork().getIShellFolder() && getAttributes0(n, n2, n3) == n3) {
            String fileSystemPath = getFileSystemPath(Win32ShellFolderManager2.getDesktop().getIShellFolder(), getLinkLocation(n, n2, false));
            if (fileSystemPath != null && fileSystemPath.startsWith("\\\\")) {
                return fileSystemPath;
            }
        }
        return getDisplayNameOf(n, n2, 32768);
    }
    
    static String getFileSystemPath(final int n) throws IOException, InterruptedException {
        String file = (String)ShellFolder.invoke(new Callable<String>() {
            @Override
            public String call() throws IOException {
                return Win32ShellFolder2.access$800(n);
            }
        }, IOException.class);
        if (file != null) {
            SecurityManager securityManager = System.getSecurityManager();
            if (securityManager != null) {
                securityManager.checkRead(file);
            }
        }
        return file;
    }
    
    private static native String getFileSystemPath0(int p0) throws IOException;
    
    private static boolean isNetworkRoot(String s) {
        return s.equals("\\\\") || s.equals("\\") || s.equals("//") || s.equals("/");
    }
    
    @Override
    public File getParentFile() {
        return this.parent;
    }
    
    @Override
    public boolean isDirectory() {
        if (this.isDir == null) {
            if (this.hasAttribute(536870912) && !this.hasAttribute(134217728)) {
                this.isDir = Boolean.TRUE;
            }
            else if (this.isLink()) {
                ShellFolder linkLocation = this.getLinkLocation(false);
                this.isDir = Boolean.valueOf(linkLocation != null && linkLocation.isDirectory());
            }
            else {
                this.isDir = Boolean.FALSE;
            }
        }
        return (boolean)this.isDir;
    }
    
    private long getEnumObjects(final boolean b) throws InterruptedException {
        return (long)ShellFolder.invoke(new Callable<Long>() {
            @Override
            public Long call() {
                return Long.valueOf(Win32ShellFolder2.access$900(Win32ShellFolder2.this, Win32ShellFolder2.this.disposer.pIShellFolder, Win32ShellFolder2.this.disposer.pIShellFolder == Win32ShellFolder2.this.getDesktopIShellFolder(), b));
            }
        }, RuntimeException.class);
    }
    
    private native long getEnumObjects(long p0, boolean p1, boolean p2);
    
    private native long getNextChild(long p0);
    
    private native void releaseEnumObjects(long p0);
    
    private static native long bindToObject(long p0, long p1);
    
    @Override
    public File[] listFiles(final boolean b) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkRead(this.getPath());
        }
        try {
            return Win32ShellFolderManager2.checkFiles((File[])ShellFolder.invoke(new Callable<File[]>() {
                @Override
                public File[] call() throws InterruptedException {
                    if (!Win32ShellFolder2.this.isDirectory()) {
                        return null;
                    }
                    if (Win32ShellFolder2.this.isLink() && !Win32ShellFolder2.this.hasAttribute(536870912)) {
                        return new File[0];
                    }
                    Win32ShellFolder2 desktop = Win32ShellFolderManager2.getDesktop();
                    Win32ShellFolder2 personal = Win32ShellFolderManager2.getPersonal();
                    long access$200 = Win32ShellFolder2.access$200(Win32ShellFolder2.this);
                    ArrayList<Win32ShellFolder2> list = new ArrayList();
                    long access$201 = Win32ShellFolder2.access$1000(Win32ShellFolder2.this, b);
                    if (access$201 != 0L) {
                        try {
                            int n = 1342177280;
                            long access$202;
                            do {
                                access$202 = Win32ShellFolder2.access$1100(Win32ShellFolder2.this, access$201);
                                boolean n2 = true;
                                if (access$202 != 0L && (Win32ShellFolder2.access$700(access$200, access$202, n) & n) != 0x0) {
                                    if (Win32ShellFolder2.this.equals(desktop) && personal != null && Win32ShellFolder2.access$1200(access$200, access$202, personal.disposer.relativePIDL)) {
                                    }
                                    else {
                                        personal = new Win32ShellFolder2(Win32ShellFolder2.this, access$202);
                                        n2 = false;
                                    }
                                    list.add(personal);
                                }
                                if (!(n2 == 0)) {
                                    Win32ShellFolder2.releasePIDL(access$202);
                                }
                                else {
                                    continue;
                                }
                            } while (access$202 != 0L && !Thread.currentThread().isInterrupted());
                        }
                        finally {
                            Win32ShellFolder2.access$1300(Win32ShellFolder2.this, access$201);
                        }
                    }
                    return Thread.currentThread().isInterrupted() ? new File[0] : ((File[])list.toArray(new ShellFolder[list.size()]));
                }
            }, InterruptedException.class));
        }
        catch (InterruptedException ex) {
            return new File[0];
        }
    }
    
    Win32ShellFolder2 getChildByPath(final String s) throws InterruptedException {
        return (Win32ShellFolder2)ShellFolder.invoke(new Callable<Win32ShellFolder2>() {
            @Override
            public Win32ShellFolder2 call() throws InterruptedException {
                long access$200 = Win32ShellFolder2.access$200(Win32ShellFolder2.this);
                long access$201 = Win32ShellFolder2.access$1000(Win32ShellFolder2.this, true);
                Win32ShellFolder2 win32ShellFolder2 = null;
                long access$202;
                while ((access$202 = Win32ShellFolder2.access$1100(Win32ShellFolder2.this, access$201)) != 0L) {
                    if (Win32ShellFolder2.access$700(access$200, access$202, 1073741824) != 0) {
                        String access$203 = Win32ShellFolder2.access$400(access$200, access$202);
                        if (access$203 != null && access$203.equalsIgnoreCase(s)) {
                            win32ShellFolder2 = new Win32ShellFolder2(Win32ShellFolder2.this, Win32ShellFolder2.access$500(access$200, access$202), access$202, access$203);
                            break;
                        }
                    }
                    Win32ShellFolder2.releasePIDL(access$202);
                }
                Win32ShellFolder2.access$1300(Win32ShellFolder2.this, access$201);
                return win32ShellFolder2;
            }
        }, InterruptedException.class);
    }
    
    @Override
    public boolean isLink() {
        if (this.cachedIsLink == null) {
            this.cachedIsLink = Boolean.valueOf(this.hasAttribute(65536));
        }
        return (boolean)this.cachedIsLink;
    }
    
    @Override
    public boolean isHidden() {
        return this.hasAttribute(524288);
    }
    
    private static native long getLinkLocation(long p0, long p1, boolean p2);
    
    @Override
    public ShellFolder getLinkLocation() {
        return this.getLinkLocation(true);
    }
    
    private ShellFolder getLinkLocation(final boolean b) {
        return (ShellFolder)ShellFolder.invoke(new Callable<ShellFolder>() {
            @Override
            public ShellFolder call() {
                if (!Win32ShellFolder2.this.isLink()) {
                    return null;
                }
                Win32ShellFolder2 shellFolderFromRelativePIDL = null;
                long access$1400 = Win32ShellFolder2.access$1400(Win32ShellFolder2.this.getParentIShellFolder(), Win32ShellFolder2.this.getRelativePIDL(), b);
                if (access$1400 != 0L) {
                    try {
                        shellFolderFromRelativePIDL = Win32ShellFolderManager2.createShellFolderFromRelativePIDL(Win32ShellFolder2.this.getDesktop(), access$1400);
                    }
                    catch (InterruptedException ex) {}
                    catch (InternalError internalError) {}
                }
                return shellFolderFromRelativePIDL;
            }
        });
    }
    
    long parseDisplayName(final String s) throws IOException, InterruptedException {
        return (long)ShellFolder.invoke(new Callable<Long>() {
            @Override
            public Long call() throws IOException {
                return Long.valueOf(Win32ShellFolder2.access$1500(Win32ShellFolder2.access$200(Win32ShellFolder2.this), s));
            }
        }, IOException.class);
    }
    
    private static native long parseDisplayName0(long p0, String p1) throws IOException;
    
    private static native String getDisplayNameOf(long p0, long p1, int p2);
    
    @Override
    public String getDisplayName() {
        if (this.displayName == null) {
            this.displayName = (String)ShellFolder.invoke(new Callable<String>() {
                @Override
                public String call() {
                    return Win32ShellFolder2.access$1600(Win32ShellFolder2.this.getParentIShellFolder(), Win32ShellFolder2.this.getRelativePIDL(), 0);
                }
            });
        }
        return this.displayName;
    }
    
    private static native String getFolderType(long p0);
    
    @Override
    public String getFolderType() {
        if (this.folderType == null) {
            this.folderType = (String)ShellFolder.invoke(new Callable<String>() {
                
                @Override
                public String call() {
                    return Win32ShellFolder2.access$1700(absolutePIDL);
                }
            });
        }
        return this.folderType;
    }
    
    private native String getExecutableType(String p0);
    
    @Override
    public String getExecutableType() {
        if (!this.isFileSystem()) {
            return null;
        }
        return this.getExecutableType(this.getAbsolutePath());
    }
    
    private static native long getIShellIcon(long p0);
    
    private static native int getIconIndex(long p0, long p1);
    
    private static native long getIcon(String p0, boolean p1);
    
    private static native long extractIcon(long p0, long p1, boolean p2);
    
    private static native long getSystemIcon(int p0);
    
    private static native long getIconResource(String p0, int p1, int p2, int p3, boolean p4);
    
    private static native int[] getIconBits(long p0, int p1);
    
    private static native void disposeIcon(long p0);
    
    static native int[] getStandardViewButton0(int p0);
    
    private long getIShellIcon() {
        if (this.pIShellIcon == -1L) {
            this.pIShellIcon = getIShellIcon(this.getIShellFolder());
        }
        return this.pIShellIcon;
    }
    
    private static Image makeIcon(long n, boolean b) {
        if (n != 0L && n != -1L) {
            int n2 = b ? 32 : 16;
            int[] iconBits = getIconBits(n, n2);
            if (iconBits != null) {
                BufferedImage bufferedImage = new BufferedImage(n2, n2, 2);
                bufferedImage.setRGB(0, 0, n2, n2, iconBits, 0, n2);
                return bufferedImage;
            }
        }
        return null;
    }
    
    @Override
    public Image getIcon(final boolean b) {
        Image image = b ? this.largeIcon : this.smallIcon;
        if (image == null) {
            image = (Image)ShellFolder.invoke(new Callable<Image>() {
                @Override
                public Image call() {
                    Image image = null;
                    if (Win32ShellFolder2.this.isFileSystem()) {
                        int access$1900 = Win32ShellFolder2.access$1900((Win32ShellFolder2.this.parent != null) ? Win32ShellFolder2.access$1800((Win32ShellFolder2)Win32ShellFolder2.this.parent) : 0L, Win32ShellFolder2.this.getRelativePIDL());
                        if (access$1900 > 0) {
                            Map<Integer, Image> map;
                            if (Win32ShellFolder2.this.isLink()) {
                                map = (b ? Win32ShellFolder2.largeLinkedSystemImages : Win32ShellFolder2.smallLinkedSystemImages);
                            }
                            else {
                                map = (b ? Win32ShellFolder2.largeSystemImages : Win32ShellFolder2.smallSystemImages);
                            }
                            image = (Image)map.get(Integer.valueOf(access$1900));
                            if (image == null) {
                                long access$1901 = Win32ShellFolder2.access$2400(Win32ShellFolder2.this.getAbsolutePath(), b);
                                image = Win32ShellFolder2.access$2500(access$1901, b);
                                Win32ShellFolder2.access$2600(access$1901);
                                if (image != null) {
                                    map.put(Integer.valueOf(access$1900), image);
                                }
                            }
                        }
                    }
                    if (image == null) {
                        long access$1902 = Win32ShellFolder2.access$2700(Win32ShellFolder2.this.getParentIShellFolder(), Win32ShellFolder2.this.getRelativePIDL(), b);
                        image = Win32ShellFolder2.access$2500(access$1902, b);
                        Win32ShellFolder2.access$2600(access$1902);
                    }
                    if (image == null) {
                        image = Win32ShellFolder2.access$2801(Win32ShellFolder2.this, b);
                        return image;
                    }
                    return image;
                }
            });
            if (b) {
                this.largeIcon = image;
            }
            else {
                this.smallIcon = image;
            }
        }
        return image;
    }
    
    static Image getSystemIcon(Win32ShellFolder2.SystemIcon systemIcon) {
        long systemIcon2 = getSystemIcon(systemIcon.getIconID());
        Image icon = makeIcon(systemIcon2, true);
        disposeIcon(systemIcon2);
        return icon;
    }
    
    static Image getShell32Icon(int n, boolean b) {
        boolean equals = true;
        int n2 = b ? 32 : 16;
        String s = (String)Toolkit.getDefaultToolkit().getDesktopProperty("win.icon.shellIconBPP");
        if (s != null) {
            equals = s.equals("4");
        }
        long iconResource = getIconResource("shell32.dll", n, n2, n2, (boolean)equals);
        if (iconResource != 0L) {
            Image icon = makeIcon(iconResource, b);
            disposeIcon(iconResource);
            return icon;
        }
        return null;
    }
    
    @Override
    public File getCanonicalFile() throws IOException {
        return this;
    }
    
    public boolean isSpecial() {
        return this.isPersonal || !this.isFileSystem() || this == this.getDesktop();
    }
    
    @Override
    public int compareTo(File file) {
        if (!(file instanceof Win32ShellFolder2)) {
            if (this.isFileSystem() && !this.isSpecial()) {
                return super.compareTo(file);
            }
            return -1;
        }
        else {
            return Win32ShellFolderManager2.compareShellFolders(this, (Win32ShellFolder2)file);
        }
    }
    
    @Override
    public ShellFolderColumnInfo[] getFolderColumns() {
        return (ShellFolderColumnInfo[])ShellFolder.invoke(new Callable<ShellFolderColumnInfo[]>() {
            @Override
            public ShellFolderColumnInfo[] call() {
                ShellFolderColumnInfo[] access$2900 = Win32ShellFolder2.access$2900(Win32ShellFolder2.this, Win32ShellFolder2.access$200(Win32ShellFolder2.this));
                if (access$2900 != null) {
                    ArrayList<ShellFolderColumnInfo> list = new ArrayList();
                    for (int i = 0; i < access$2900.length; ++i) {
                        ShellFolderColumnInfo shellFolderColumnInfo = access$2900[i];
                        if (shellFolderColumnInfo != null) {
                            shellFolderColumnInfo.setAlignment(Integer.valueOf(((int)shellFolderColumnInfo.getAlignment() == 1) ? 4 : (((int)shellFolderColumnInfo.getAlignment() == 2) ? 0 : 10)));
                            shellFolderColumnInfo.setComparator(new Win32ShellFolder2.ColumnComparator(Win32ShellFolder2.this, i));
                            list.add(shellFolderColumnInfo);
                        }
                    }
                    access$2900 = new ShellFolderColumnInfo[list.size()];
                    list.toArray(access$2900);
                    return access$2900;
                }
                return access$2900;
            }
        });
    }
    
    @Override
    public Object getFolderColumnValue(final int n) {
        return ShellFolder.invoke(new Callable<Object>() {
            @Override
            public Object call() {
                return Win32ShellFolder2.access$3000(Win32ShellFolder2.this, Win32ShellFolder2.this.getParentIShellFolder(), Win32ShellFolder2.this.getRelativePIDL(), n);
            }
        });
    }
    
    private native ShellFolderColumnInfo[] doGetColumnInfo(long p0);
    
    private native Object doGetColumnValue(long p0, long p1, int p2);
    
    private static native int compareIDsByColumn(long p0, long p1, long p2, int p3);
    
    @Override
    public void sortChildren(final List<? extends File> list) {
        ShellFolder.invoke(new Callable<Void>() {
            @Override
            public Void call() {
                Collections.sort(list, new Win32ShellFolder2.ColumnComparator(Win32ShellFolder2.this, 0));
                return null;
            }
        });
    }
    
    static void access$000(long n) {
        releaseIShellFolder(n);
    }
    
    static void access$100(Win32ShellFolder2 win32ShellFolder2) {
        win32ShellFolder2.initDesktop();
    }
    
    static long access$200(Win32ShellFolder2 win32ShellFolder2) {
        return win32ShellFolder2.getIShellFolder();
    }
    
    static void access$300(Win32ShellFolder2 win32ShellFolder2, long n, int n2) {
        win32ShellFolder2.initSpecial(n, n2);
    }
    
    static String access$400(long n, long n2) {
        return getFileSystemPath(n, n2);
    }
    
    static long access$500(long n, long n2) {
        return bindToObject(n, n2);
    }
    
    static int access$600(long n, long n2, long n3) {
        return compareIDs(n, n2, n3);
    }
    
    static int access$700(long n, long n2, int n3) {
        return getAttributes0(n, n2, n3);
    }
    
    static String access$800(int n) throws IOException {
        return getFileSystemPath0(n);
    }
    
    static long access$900(Win32ShellFolder2 win32ShellFolder2, long n, boolean b, boolean b2) {
        return win32ShellFolder2.getEnumObjects(n, b, b2);
    }
    
    static long access$1000(Win32ShellFolder2 win32ShellFolder2, boolean b) throws InterruptedException {
        return win32ShellFolder2.getEnumObjects(b);
    }
    
    static long access$1100(Win32ShellFolder2 win32ShellFolder2, long n) {
        return win32ShellFolder2.getNextChild(n);
    }
    
    static boolean access$1200(long n, long n2, long n3) throws InterruptedException {
        return pidlsEqual(n, n2, n3);
    }
    
    static void access$1300(Win32ShellFolder2 win32ShellFolder2, long n) {
        win32ShellFolder2.releaseEnumObjects(n);
    }
    
    static long access$1400(long n, long n2, boolean b) {
        return getLinkLocation(n, n2, b);
    }
    
    static long access$1500(long n, String s) throws IOException {
        return parseDisplayName0(n, s);
    }
    
    static String access$1600(long n, long n2, int n3) {
        return getDisplayNameOf(n, n2, n3);
    }
    
    static String access$1700(long n) {
        return getFolderType(n);
    }
    
    static long access$1800(Win32ShellFolder2 win32ShellFolder2) {
        return win32ShellFolder2.getIShellIcon();
    }
    
    static int access$1900(long n, long n2) {
        return getIconIndex(n, n2);
    }
    
    static long access$2400(String s, boolean b) {
        return getIcon(s, b);
    }
    
    static Image access$2500(long n, boolean b) {
        return makeIcon(n, b);
    }
    
    static void access$2600(long n) {
        disposeIcon(n);
    }
    
    static long access$2700(long n, long n2, boolean b) {
        return extractIcon(n, n2, b);
    }
    
    static Image access$2801(Win32ShellFolder2 win32ShellFolder2, boolean b) {
        return win32ShellFolder2.getIcon(b);
    }
    
    static ShellFolderColumnInfo[] access$2900(Win32ShellFolder2 win32ShellFolder2, long n) {
        return win32ShellFolder2.doGetColumnInfo(n);
    }
    
    static Object access$3000(Win32ShellFolder2 win32ShellFolder2, long n, long n2, int n3) {
        return win32ShellFolder2.doGetColumnValue(n, n2, n3);
    }
    
    static int access$3300(long n, long n2, long n3, int n4) {
        return compareIDsByColumn(n, n2, n3, n4);
    }
    
    static {
        initIDs();
        Win32ShellFolder2.smallSystemImages = new HashMap();
        Win32ShellFolder2.largeSystemImages = new HashMap();
        Win32ShellFolder2.smallLinkedSystemImages = new HashMap();
        Win32ShellFolder2.largeLinkedSystemImages = new HashMap();
    }
    
    public enum SystemIcon
    {
        IDI_APPLICATION(32512), 
        IDI_HAND(32513), 
        IDI_ERROR(32513), 
        IDI_QUESTION(32514), 
        IDI_EXCLAMATION(32515), 
        IDI_WARNING(32515), 
        IDI_ASTERISK(32516), 
        IDI_INFORMATION(32516), 
        IDI_WINLOGO(32517);
        
        private final int iconID;
        
        private SystemIcon(int iconID) {
            this.iconID = iconID;
        }
        
        public int getIconID() {
            return this.iconID;
        }
    }
    
    static class FolderDisposer implements DisposerRecord
    {
        long absolutePIDL;
        long pIShellFolder;
        long relativePIDL;
        boolean disposed;
        
        @Override
        public void dispose() {
            if (this.disposed) {
                return;
            }
            ShellFolder.invoke(new Callable<Void>() {
                @Override
                public Void call() {
                    if (Win32ShellFolder2.FolderDisposer.this.relativePIDL != 0L) {
                        Win32ShellFolder2.releasePIDL(Win32ShellFolder2.FolderDisposer.this.relativePIDL);
                    }
                    if (Win32ShellFolder2.FolderDisposer.this.absolutePIDL != 0L) {
                        Win32ShellFolder2.releasePIDL(Win32ShellFolder2.FolderDisposer.this.absolutePIDL);
                    }
                    if (Win32ShellFolder2.FolderDisposer.this.pIShellFolder != 0L) {
                        Win32ShellFolder2.access$000(Win32ShellFolder2.FolderDisposer.this.pIShellFolder);
                    }
                    return null;
                }
            });
            this.disposed = true;
            return;
        }
    }
    
    private static class ColumnComparator implements Comparator<File>
    {
        private final Win32ShellFolder2 shellFolder;
        private final int columnIdx;
        
        public ColumnComparator(Win32ShellFolder2 shellFolder, int columnIdx) {
            this.shellFolder = shellFolder;
            this.columnIdx = columnIdx;
        }
        
        @Override
        public int compare(File file, File file2) {
            Integer n = (Integer)ShellFolder.invoke(new Callable<Integer>() {
                @Override
                public Integer call() {
                    if (file instanceof Win32ShellFolder2 && file2 instanceof Win32ShellFolder2) {
                        return Integer.valueOf(Win32ShellFolder2.access$3300(Win32ShellFolder2.access$200(Win32ShellFolder2.ColumnComparator.this.shellFolder), ((Win32ShellFolder2)file).getRelativePIDL(), ((Win32ShellFolder2)file2).getRelativePIDL(), Win32ShellFolder2.ColumnComparator.this.columnIdx));
                    }
                    return Integer.valueOf(0);
                }
            });
            return (n == null) ? 0 : ((int)n);
        }
    }
}
